// Contains enums and messages reused throught the Microgrid API definitions.
//
// Copyright:
// Copyright 2022 Frequenz Energy-as-a-Service GmbH
//
// License:
// MIT

syntax = "proto3";

package frequenz.api.microgrid.common;

// Error levels definitions.
enum ErrorLevel {
  // Unspecified component error.
  ERROR_LEVEL_UNSPECIFIED = 0;

  // Action must be taken to prevent a severe error from occurring in the
  // future.
  ERROR_LEVEL_WARN = 1;

  // A severe error that causes the component to fail. Immediate action must be
  // taken.
  ERROR_LEVEL_CRITICAL = 2;
}
