// Contains grid connection point details.
//
// Copyright:
// Copyright 2023 Frequenz Energy-as-a-Service GmbH
//
// License:
// MIT

syntax = "proto3";

package frequenz.api.microgrid.grid;

// The grid connection point metadata.
message Metadata {
  // This refers to the maximum amount of electrical current, in amperes, that a
  // fuse at the grid connection point is designed to safely carry under normal
  // operating conditions.
  //
  // This limit applies to currents both flowing in or out of each of the 3
  // phases individually.
  //
  // In other words, a current _i_ A at one of the phases of the grid connection
  // point must comply with the following constraint:
  // `-rated_fuse_current <= i <= rated_fuse_current`
  uint32 rated_fuse_current = 1;
}
